-- Per-character settings
chatEnabled = false;
floatingEnabled = true;
displayEnabled = true;
killsRemaining = 0;
questsRemaining = 0;

-- Per session settings
local xpFromMobsTotal = 0;
local mobsKilled = 0;
local xpFromQuestsTotal = 0;
local questsCompleted = 0;
local playerXP, playerXPMax, playerXPRested;

-- GUI
local waterfall = AceLibrary("Waterfall-1.0")

-- Initialize
function XToLevel_OnLoad()
	-- Check if player is max level
	if(UnitLevel("player") == 70) then
		return;
	end
	
	-- Check if the client language is suported
	if(GetLocale() ~= "enUS") then
		DEFAULT_CHAT_FRAME:AddMessage("XToLevel could not be loaded. Client language is not supported.");
		return;
	end

	-- Register Events
	this:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN");
	this:RegisterEvent("PLAYER_LOGIN");
	this:RegisterEvent("PLAYER_LEVEL_UP");
	
	-- Initialize GUI config
	ConfigWindow_Init();
	
	-- Register slash commands
	SLASH_XTOLEVEL1 = "/xtolevel";
	SLASH_XTOLEVEL2 = "/xtl";
	SlashCmdList["XTOLEVEL"] = SlashHandler;
end

function ConfigWindow_Init()
	waterfall:Register('ConfigWindow',
		'title','XToLevel Config',
		'colorR', 0.0,
		'colorG', 0.5,
		'colroB', 0.25,
		'tree', 
		{
			{
				text = 'Display',
				id = 'display',
				isOpen = true,
				isSelected = true
			}
		},
	 'children', 
		function(id)
			if id=='display' then
				waterfall:AddControl(
					'type', 'checkbox',
					'text', "Show in chat window",
					'checked', chatEnabled,
					'fullRefresh', true,
					'setFunc', function() chatEnabled = not chatEnabled end
		        )
				waterfall:AddControl(
					'type', 'checkbox',
					'text', "Show floating message",
					'checked', floatingEnabled,
					'fullRefresh', true,
					'setFunc', function() floatingEnabled = not floatingEnabled end
		        )
				waterfall:AddControl(
					'type', 'checkbox',
					'text', "Enable avarage display",
					'checked', displayEnabled,
					'fullRefresh', true,
					'setFunc', ToogleDisplay
		        )
				return 'Display Config'
			end
		end 
	)
end

-- Slash Command handler
function SlashHandler()
	waterfall:Open('ConfigWindow', 'display')
end

-- Dispay handler
function ToogleDisplay()
	displayEnabled = not displayEnabled;
	if displayEnabled then
		XToLevelDisplay:Show();
	else
		XToLevelDisplay:Hide();
	end
end

-- Event control
function XToLevel_OnEvent()
	if (event == "PLAYER_LOGIN") then
		-- Set display
		XToLevelDisplayLabelText:SetText("XToLevel:");
		if(killsRemaining == 0 and questsRemaining == 0) then
			XToLevelDisplayKillText:SetText(" Waiting...");
			XToLevelDisplayQuestText:SetText("");
		elseif (questsRemaining == 0) then
			XToLevelDisplayKillText:SetText(" Kills: ".. killsRemaining);
			XToLevelDisplayQuestText:SetText("");
		else
			XToLevelDisplayKillText:SetText(" Kills: ".. killsRemaining);
			XToLevelDisplayQuestText:SetText(" Quests: ".. questsRemaining);
		end
		
		-- Show / Hide display
		if displayEnabled then
			XToLevelDisplay:Show();
		else
			XToLevelDisplay:Hide();
		end
	elseif (event == "PLAYER_LEVEL_UP") then
		UpdateEstimates();
	elseif (event == "CHAT_MSG_COMBAT_XP_GAIN") then
		local msg = "n/a"
			
		-- Gather the XP data
		playerXP = UnitXP("player")
		playerXPMax = UnitXPMax("player")
		playerXPRested = GetXPExhaustion() or 0
		
		-- Check if the rested bonus extends beiond the next level
		if(playerXP + playerXPRested > playerXPMax) then
			playerXPRested = (playerXPMax - playerXP)
		end
		
		-- Extract the info from the message
		local mobName, xpGained = ExtractKillInfoReg(arg1);
		xpGained = tonumber(xpGained)
		
		-- Level reached
		if(playerXP + xpGained > playerXPMax) then
			msg = "Level reached. Congratulations"
		-- Quest turn-in
		elseif(mobName == nil) then
			-- Calculate the ammount of quest turnins to level
			--  (TotalLevelXP - XPSoFar - GainedXP) / GainedXP
			local quests = ceil((playerXPMax - playerXP - xpGained) / xpGained)
			
			-- Create message
			msg = quests .. " more quests like that needed to level"
			
			-- Set per session data
			xpFromQuestsTotal = xpFromQuestsTotal + xpGained;
			questsCompleted = questsCompleted + 1;
			
			-- Set the display
			UpdateEstimates();
			
		-- Mob kill
		else
			-- Check if the XP gained was rested and remove the rested bonus if needed
			if(playerXPRested > 0) then
				if(playerXPRested * 2 < xpGained) then
					xpGained = xpGained - playerXPRested
				else
					xpGained = xpGained / 2
				end
			end
			
			-- Get How much rested till next level
			local xpToLevel = playerXPMax - playerXP;
			local restedToLevel = 0;
			if(playerXPRested == 0) then
				restedToLevel = 0;
			elseif(xpToLevel < playerXPRested) then
				restedToLevel = xpToLevel;
			else
				restedToLevel = playerXPRested;
			end
			
			-- Calculate the ammount of mobs needed
			--  (TotalLevelXP - XPSoFar - (RestedBonus)) / GainedXP
			local kills = ceil((xpToLevel - (restedToLevel / 2) - xpGained) / xpGained)
			
			-- Create Message
			msg = kills .." ".. mobName .." needed to level"
			
			-- Set per session data
			xpFromMobsTotal = xpFromMobsTotal + xpGained;
			mobsKilled = mobsKilled + 1;
			
			-- Set the display
			UpdateEstimates();
		end
			
		-- Send Messages
		if chatEnabled then
			DEFAULT_CHAT_FRAME:AddMessage(msg, 0, 100, 200, 55, 4);
		end
		if floatingEnabled then
			UIErrorsFrame:AddMessage(msg, 0, 100, 200, 55, 4);
		end
	end
end

function ExtractKillInfoReg(message)
	local regEx, mob, xp;
	--- Check if this is a quest or a mob
	if(string.find(message, ",") == nil) then
		regEx = "You gain (%d+) experience";
		xp = strmatch(message, regEx);
	else
		local regEx = "(.+) dies, you gain (%d+) experience";
		if(GetLocale() == "deDE") then
			regEx = "(.+) stirbt, Ihr bekommt (%d+) Erfahrung";
		elseif (GetLocale() == "frFR") then
			regEx = "(.+) succombe, vous gagnez (%d+) points d'experience";
		end
		mob, xp = strmatch(message, regEx);
	end
	
	return mob, xp;
end

function UpdateEstimates()
	-- Get How much rested till next level
	local xpToLevel = playerXPMax - playerXP;
	local restedToLevel = 0;
	if(playerXPRested == 0) then
		restedToLevel = 0;
	elseif(xpToLevel < playerXPRested) then
		restedToLevel = xpToLevel;
	else
		restedToLevel = playerXPRested;
	end

	-- Set Kill avrg
	if(xpFromMobsTotal ~= 0 and mobsKilled ~= 0) then
		local avrgPerKill = ceil(xpFromMobsTotal / mobsKilled);
		killsRemaining = ceil((xpToLevel - (restedToLevel / 2) - avrgPerKill) / avrgPerKill);
		XToLevelDisplayKillText:SetText(" Kills: ".. killsRemaining);
	elseif(killsRemaining ~= 0) then
		XToLevelDisplayKillText:SetText(" Kills: ".. killsRemaining);
	end
	
	-- Set quest avrg
	if(xpFromQuestsTotal ~= 0 and questsCompleted ~= 0) then
		local avrgPerQuest = ceil(xpFromQuestsTotal / questsCompleted);
		questsRemaining = ceil((xpToLevel - avrgPerQuest) / avrgPerQuest);
		XToLevelDisplayQuestText:SetText(" Quests: ".. questsRemaining);
	elseif(questsRemaining ~= 0) then
		XToLevelDisplayQuestText:SetText(" Quests: ".. questsRemaining);
	end
end
